from collections.abc import Callable
from functools import wraps
from typing import ParamSpec, TypeVar

from starbridge.base import __project_name__

from .models import PromptMetadata, ResourceMetadata, ToolMetadata

P = ParamSpec("P")
R = TypeVar("R")


def mcp_tool(
    server: str = __project_name__, service: str | None = None, name: str | None = None
) -> Callable[[Callable[P, R]], Callable[P, R]]:
    """Decorator to mark a method as an MCP tool.

    Args:
        server (str, optional): The server name. Defaults to "starbridge".
        service (str, optional): The service name. If not provided, derived from module name.
        name (str, optional): The tool name. If not provided, derived from function name.
    """

    def decorator(func: Callable[P, R]) -> Callable[P, R]:
        @wraps(func)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        wrapper.__mcp_tool__ = ToolMetadata(  # type: ignore
            server=server,
            service=service or _get_service_name(func),
            name=name or func.__name__,
        )
        return wrapper

    return decorator


def mcp_resource_iterator(
    server: str = __project_name__, service: str | None = None, type: str | None = None
):
    """Decorator to mark a method as a resource iterator."""

    def decorator(func: Callable):
        @wraps(func)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        wrapper.__mcp_resource_iterator__ = ResourceMetadata(  # type: ignore
            server=server,
            service=service or _get_service_name(func),
            type=type or "",
        )
        return wrapper

    return decorator


def mcp_resource(
    server: str = __project_name__, service: str | None = None, type: str | None = None
):
    """Decorator to mark a method as a resource handler."""

    def decorator(func: Callable):
        @wraps(func)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        wrapper.__mcp_resource__ = ResourceMetadata(  # type: ignore
            server=server,
            service=service or _get_service_name(func),
            type=type or "",
        )
        return wrapper

    return decorator


def mcp_prompt(
    server: str = __project_name__, service: str | None = None, type: str | None = None
):
    """Decorator to mark a method as a prompt handler."""

    def decorator(func: Callable):
        @wraps(func)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        wrapper.__mcp_prompt__ = PromptMetadata(  # type: ignore
            server=server,
            service=service or _get_service_name(func),
            type=type or func.__name__.replace("prompt_", ""),
        )
        return wrapper

    return decorator


def _get_service_name(func: Callable) -> str:
    """Extract service name from function's module path."""
    return func.__module__.split(".")[1]



"""Original source: https://github.com/pydantic/pydantic-ai/blob/main/pydantic_ai_slim/pydantic_ai/_griffe.py"""

from __future__ import annotations as _annotations

import re
from collections.abc import Callable
from inspect import Parameter, signature
from typing import Any, Literal, cast

from _griffe.enumerations import DocstringSectionKind
from _griffe.models import Docstring
from _griffe.models import Object as GriffeObject

DocstringStyle = Literal["google", "numpy", "sphinx"]


def description_and_params(
    func: Callable[..., Any], excluded_params=("self", "context")
) -> tuple[str, list, dict[str, dict]]:
    """Extract the function description and parameter descriptions from a function's docstring.

    Returns:
        A tuple of (main function description, parameter properties dict).
    """
    doc = func.__doc__
    if doc is None:
        return func.__name__, [], {}

    sig = signature(func)

    # see https://github.com/mkdocstrings/griffe/issues/293
    parent = cast(GriffeObject, sig)

    docstring = Docstring(
        doc, lineno=1, parser=_infer_docstring_style(doc), parent=parent
    )
    sections = docstring.parse()

    # Get parameter descriptions from docstring
    param_desc = {}
    if parameters := next(
        (p for p in sections if p.kind == DocstringSectionKind.parameters), None
    ):
        param_desc = {p.name: p.description for p in parameters.value}

    # Build parameter properties including type info
    params = {}
    required = []
    for param in sig.parameters.values():
        if param.name in excluded_params:
            continue

        param_type = "string"  # default type
        if param.annotation != Parameter.empty:
            if param.annotation is str:
                param_type = "string"
            elif param.annotation is int:
                param_type = "number"
            elif param.annotation is bool:
                param_type = "boolean"

        if param.default == Parameter.empty:
            required.append(param.name)

        params[param.name] = {
            "type": param_type,
            "description": param_desc.get(param.name, f"Parameter {param.name}"),
        }

    main_desc = ""
    if main := next((p for p in sections if p.kind == DocstringSectionKind.text), None):
        main_desc = main.value

    return main_desc, required, params


def _infer_docstring_style(doc: str) -> DocstringStyle:
    """Simplistic docstring style inference."""
    for pattern, replacements, style in _docstring_style_patterns:
        matches = (
            re.search(pattern.format(replacement), doc, re.IGNORECASE | re.MULTILINE)
            for replacement in replacements
        )
        if any(matches):
            return style
    # fallback to google style
    return "google"


# See https://github.com/mkdocstrings/griffe/issues/329#issuecomment-2425017804
_docstring_style_patterns: list[tuple[str, list[str], DocstringStyle]] = [
    (
        r"\n[ \t]*:{0}([ \t]+\w+)*:([ \t]+.+)?\n",
        [
            "param",
            "parameter",
            "arg",
            "argument",
            "key",
            "keyword",
            "type",
            "var",
            "ivar",
            "cvar",
            "vartype",
            "returns",
            "return",
            "rtype",
            "raises",
            "raise",
            "except",
            "exception",
        ],
        "sphinx",
    ),
    (
        r"\n[ \t]*{0}:([ \t]+.+)?\n[ \t]+.+",
        [
            "args",
            "arguments",
            "params",
            "parameters",
            "keyword args",
            "keyword arguments",
            "other args",
            "other arguments",
            "other params",
            "other parameters",
            "raises",
            "exceptions",
            "returns",
            "yields",
            "receives",
            "examples",
            "attributes",
            "functions",
            "methods",
            "classes",
            "modules",
            "warns",
            "warnings",
        ],
        "google",
    ),
    (
        r"\n[ \t]*{0}\n[ \t]*---+\n",
        [
            "deprecated",
            "parameters",
            "other parameters",
            "returns",
            "yields",
            "receives",
            "raises",
            "warns",
            "attributes",
            "functions",
            "methods",
            "classes",
            "modules",
        ],
        "numpy",
    ),
]



import asyncio
import base64
import json
import os
from io import BytesIO
from typing import Any
from urllib.parse import urlparse

import mcp.server.stdio
import mcp.types as types
import pydantic_core
from mcp.server import NotificationOptions, Server
from mcp.server.models import InitializationOptions
from mcp.server.sse import SseServerTransport
from mcp.types import (
    EmbeddedResource,
    ImageContent,
    TextContent,
)
from PIL import Image
from pydantic import AnyUrl
from starlette.applications import Starlette
from starlette.responses import PlainTextResponse
from starlette.routing import Route

from starbridge.base import __project_name__, __version__
from starbridge.mcp.context import MCPContext
from starbridge.mcp.decorators import mcp_tool
from starbridge.mcp.models import ResourceMetadata
from starbridge.mcp.service import MCPBaseService
from starbridge.utils import get_logger

logger = get_logger(__name__)


class MCPServer(MCPBaseService):
    """MCP Server for Starbridge."""

    def __init__(self):
        self._services = []
        for service_class in MCPBaseService.get_services():
            self._services.append(service_class())

        self._server = Server(__project_name__)
        self._server.list_prompts()(self.prompt_list)
        self._server.get_prompt()(self.prompt_get)
        self._server.list_resources()(self.resource_list)
        self._server.read_resource()(self.resource_get)
        self._server.list_tools()(self.tool_list)
        self._server.call_tool()(self.tool_call)

    @mcp_tool()
    def health(self, context: MCPContext | None = None):
        """Health of services and their dependencies"""
        dependencies = {}
        for service_class in MCPBaseService.get_services():
            service = service_class()
            service_name = service.__class__.__module__.split(".")[1]
            dependencies[service_name] = service.health()

        healthy = all(status == "UP" for status in dependencies.values())
        return {"healthy": healthy, "dependencies": dependencies}

    def get_context(self) -> MCPContext:
        """
        Returns a Context object. Note that the context will only be valid
        during a request; outside a request, most methods will error.
        """
        try:
            request_context = self._server.request_context
        except LookupError:
            request_context = None
        return MCPContext(request_context=request_context, mcp=self)

    async def resource_list(self) -> list[types.Resource]:
        resources = []
        for service in self._services:
            resources.extend(service.resource_list(context=self.get_context()))
        return resources

    async def resource_get(self, uri: AnyUrl) -> str:
        """Get a resource from any service that can handle it."""
        parsed = urlparse(str(uri))

        # Try each service's resource handlers
        for service in self._services:
            # Find all resource handlers in the service
            for method_name in dir(service.__class__):
                method = getattr(service.__class__, method_name)
                if hasattr(method, "__mcp_resource__"):
                    meta = method.__mcp_resource__
                    # Check if this handler matches the URI pattern
                    if (
                        meta.server == parsed.scheme
                        and meta.service == parsed.netloc
                        and parsed.path.startswith(f"/{meta.type}/")
                    ):  # renamed from prefix
                        # Call the handler with the resource ID (last part of path)
                        result = method(
                            service,
                            parsed.path.split("/")[-1],
                            context=self.get_context(),
                        )
                        if result is not None:
                            return result

        raise ValueError(f"No service found for URI: {uri}")

    async def prompt_list(
        self,
    ) -> list[types.Prompt]:
        prompts = []
        for service in self._services:
            prompts.extend(service.prompt_list(context=self.get_context()))
        return prompts

    async def prompt_get(
        self, name: str, arguments: dict[str, str] | None
    ) -> types.GetPromptResult:
        """Get a prompt by its full name (server_service_type)."""
        server, service, prompt_type = name.split("_", 2)

        for service_instance in self._services:
            for method_name in dir(service_instance.__class__):
                method = getattr(service_instance.__class__, method_name)
                if hasattr(method, "__mcp_prompt__"):
                    meta = method.__mcp_prompt__
                    if (
                        meta.server == server
                        and meta.service == service
                        and meta.type == prompt_type
                    ):
                        # Found matching prompt handler
                        if arguments:
                            arguments = arguments.copy()
                            arguments.pop("context", None)
                            return method(
                                service_instance,
                                **arguments,
                                context=self.get_context(),
                            )
                        return method(service_instance, context=self.get_context())

        return types.GetPromptResult(description=None, messages=[])

    async def tool_list(
        self,
    ) -> list[types.Tool]:
        tools = []
        for service in self._services:
            tools.extend(service.tool_list(context=self.get_context()))
        return tools

    async def tool_call(
        self, name: str, arguments: dict | None
    ) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
        server, service, tool_name = name.split("_", 2)

        for service_instance in self._services:
            for method_name in dir(service_instance.__class__):
                method = getattr(service_instance.__class__, method_name)
                if hasattr(method, "__mcp_tool__"):
                    meta = method.__mcp_tool__
                    if (
                        meta.server == server
                        and meta.service == service
                        and meta.name == tool_name
                    ):
                        if arguments:
                            arguments = arguments.copy()
                            arguments.pop("context", None)
                            return MCPServer._marshal_result(
                                method(
                                    service_instance,
                                    **arguments,
                                    context=self.get_context(),
                                )
                            )
                        return MCPServer._marshal_result(
                            method(service_instance, context=self.get_context())
                        )

        raise ValueError(f"Unknown tool: {name}")

    async def resource_type_list(self) -> set[ResourceMetadata]:
        """Get all available resource types across all services."""
        types = set()
        for service in self._services:
            types.update(service.resource_type_list(context=self.get_context()))
        return types

    @staticmethod
    def resource_types() -> list[str]:
        """Get all available resource types as formatted strings."""
        return sorted(str(rt) for rt in asyncio.run(MCPServer().resource_type_list()))

    def starlette_app(self, debug: bool = True) -> Starlette:
        sse = SseServerTransport("/messages")

        async def handle_sse(request):
            async with sse.connect_sse(
                request.scope, request.receive, request._send
            ) as streams:
                await self._server.run(
                    streams[0], streams[1], self._create_initialization_options()
                )

        async def handle_messages(request):
            await sse.handle_post_message(request.scope, request.receive, request._send)

        async def handle_health(request):
            return PlainTextResponse(
                headers={"content-type": "application/json"},
                content=json.dumps(self.health()),
            )

        return Starlette(
            debug=debug,
            routes=[
                Route("/health", endpoint=handle_health, methods=["GET"]),
                Route("/sse", endpoint=handle_sse),
                Route("/messages", endpoint=handle_messages, methods=["POST"]),
            ],
        )

    async def run_stdio(self):
        async with mcp.server.stdio.stdio_server() as (read_stream, write_stream):
            await self._server.run(
                read_stream,
                write_stream,
                self._create_initialization_options(),
            )

    @staticmethod
    def services() -> list[MCPBaseService]:
        return MCPBaseService.get_services()  # type: ignore

    @staticmethod
    def tools() -> list[types.Tool]:
        return asyncio.run(MCPServer().tool_list())

    @staticmethod
    def tool(
        name: str, arguments: dict | None = None
    ) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
        return asyncio.run(MCPServer().tool_call(name, arguments))

    @staticmethod
    def resources() -> list[types.Resource]:
        return asyncio.run(MCPServer().resource_list())

    @staticmethod
    def prompts() -> list[types.Prompt]:
        return asyncio.run(MCPServer().prompt_list())

    @staticmethod
    def resource(uri: str) -> str:
        return asyncio.run(MCPServer().resource_get(AnyUrl(uri)))

    @staticmethod
    def serve(host: str | None = None, port: int | None = None, debug: bool = True):
        if host and port:
            import uvicorn

            uvicorn.run(
                MCPServer().starlette_app(debug),
                host=host,
                port=port,
                log_level=str.lower(os.environ.get("LOGLEVEL", "INFO")),
                log_config=None,
            )
        else:
            asyncio.run(MCPServer().run_stdio())

    def _create_initialization_options(self) -> InitializationOptions:
        return InitializationOptions(
            server_name=__project_name__,
            server_version=__version__,
            capabilities=self._server.get_capabilities(
                notification_options=NotificationOptions(
                    prompts_changed=False,
                    resources_changed=False,
                    tools_changed=False,
                ),
                experimental_capabilities={},
            ),
        )

    @staticmethod
    def _marshal_result(
        result: Any,
    ) -> list[TextContent | ImageContent | EmbeddedResource]:
        """Marshals a result into a sequence of TextContent, ImageContent, or EmbeddedResource."""
        if result is None:
            return []

        if isinstance(result, TextContent | ImageContent | EmbeddedResource):
            return [result]

        if isinstance(result, str):
            return [TextContent(type="text", text=result)]

        if isinstance(result, Image.Image):
            mime_type = (
                Image.MIME.get(result.format, "application/octet-stream")
                if result.format
                else "application/octet-stream"
            )
            data = BytesIO()
            result.save(data, format=result.format)
            return [
                ImageContent(
                    type="image",
                    data=base64.b64encode(data.getvalue()).decode("utf-8"),
                    mimeType=mime_type,
                )
            ]

        if isinstance(result, list | tuple):
            return [
                item
                for subresult in result
                for item in MCPServer._marshal_result(subresult)
            ]

        try:
            return [
                TextContent(
                    type="text",
                    text=json.dumps(pydantic_core.to_jsonable_python(result)),
                )
            ]
        except Exception as e:
            logger.error(f"Error converting result to JSON: {e}")

        return [TextContent(type="text", text=str(result))]



"""
CLI to interact with Confluence
"""

import os
import pathlib
import re
import subprocess
import webbrowser
from typing import Annotated

import typer

from starbridge.base import __project_name__
from starbridge.utils.console import console

from .server import MCPServer

cli = typer.Typer(no_args_is_help=True)


@cli.command()
def health():
    """Check health of the services and their dependencies."""
    console.print(MCPServer().health())


@cli.command()
def services():
    """Services exposed by modules"""
    console.print(MCPServer.services())


@cli.command()
def tools():
    """Tools exposed by modules"""
    console.print(MCPServer.tools())


@cli.command()
def tool(name: str):
    """Get tool by name"""
    console.print(MCPServer.tool(name))


@cli.command()
def resources():
    """Resources exposed by modules"""
    console.print(MCPServer.resources())


@cli.command()
def resource(uri: str):
    """Get resource by URI"""
    console.print(MCPServer.resource(uri))


@cli.command()
def prompts():
    """Prompts exposed by modules"""
    console.print(MCPServer.prompts())


@cli.command()
def resource_types():
    """Resource types exposed by modules"""
    console.print(MCPServer.resource_types())


@cli.command()
def serve(
    host: Annotated[
        str | None,
        typer.Option(
            help="Host to run the server on",
        ),
    ] = None,
    port: Annotated[
        int | None,
        typer.Option(
            help="Port to run the server on",
        ),
    ] = None,
    debug: Annotated[
        bool,
        typer.Option(
            help="Debug mode",
        ),
    ] = True,
):
    """Run MCP server."""
    MCPServer().serve(host, port, debug)


@cli.command()
def inspect():
    """Run inspector."""
    project_root = str(pathlib.Path(__file__).parent.parent.parent.parent)
    console.print(
        f"Starbridge project root: {project_root}\nStarbridge environment:\n{os.environ}"
    )
    process = subprocess.Popen(
        [
            "npx",
            "@modelcontextprotocol/inspector",
            "uv",
            "--directory",
            project_root,
            "run",
            __project_name__,
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        text=True,
    )

    url_pattern = r"MCP Inspector is up and running at (http://[^\s]+)"

    while True:
        if process.stdout is not None:
            line = process.stdout.readline()
        else:
            line = ""
        if not line:
            break
        print(line, end="")
        match = re.search(url_pattern, line)
        if match:
            url = match.group(1)
            webbrowser.open(url)

    process.wait()



"""Handles Confluence operations."""

import json

import mcp.types as types
import typer
from atlassian import Confluence
from pydantic import AnyUrl

from starbridge.atlassian.models import AtlassianSettings
from starbridge.mcp import (
    MCPBaseService,
    MCPContext,
    mcp_prompt,
    mcp_resource,
    mcp_resource_iterator,
    mcp_tool,
)
from starbridge.utils import get_logger
from starbridge.utils.settings import load_settings

from . import cli

logger = get_logger(__name__)


class Service(MCPBaseService):
    """Service class for Confluence operations."""

    def __init__(self):
        self._atlassian_settings = load_settings(AtlassianSettings)
        self._api = Confluence(
            url=str(self._atlassian_settings.url),
            username=self._atlassian_settings.email_address,
            password=self._atlassian_settings.api_token,
            cloud=True,
        )

    @staticmethod
    def get_cli() -> tuple[str | None, typer.Typer | None]:
        """Get CLI for Confluence service."""
        return "confluence", cli.cli  # type: ignore

    @mcp_tool()
    def health(self, context: MCPContext | None = None) -> str:
        try:
            spaces = self.space_list()
        except Exception as e:
            return f"DOWN: {str(e)}"
        if (
            isinstance(spaces, dict)
            and "results" in spaces
            and isinstance(spaces["results"], list)
        ):
            if len(spaces["results"]) > 0:
                return "UP"
        return "DOWN: No spaces found"

    @mcp_tool()
    def info(self, context: MCPContext | None = None):
        """Info about Confluence environment"""
        return {
            "url": self._atlassian_settings.url,
            "email_address": self._atlassian_settings.email_address,
            "api_token": f"MASKED ({len(self._atlassian_settings.api_token)} characters)",
        }

    @mcp_resource_iterator(type="space")
    def space_iterator(self, context: MCPContext | None = None):
        """List available Confluence spaces."""
        spaces = self.space_list()
        return [
            types.Resource(
                uri=AnyUrl(f"starbridge://confluence/space/{space['key']}"),
                name=space["name"],
                description=f"Space of type '{space['type']}: {space['description']}",
                mimeType="application/json",
            )
            for space in spaces["results"]
        ]

    @mcp_resource(type="space")
    def space_get(self, space_key: str, context: MCPContext | None = None) -> str:
        """Get specific Confluence space by key."""
        return json.dumps(self._api.get_space(space_key), indent=2)

    @mcp_prompt(type="space_summary")
    def space_summary(
        self,
        style: str = "brief",
        context: MCPContext | None = None,
    ) -> types.GetPromptResult:
        """Creates a summary of spaces in Confluence.

        Args:
            style: Style of the summary {'brief', 'detailed'}, defaults to 'brief'
        """
        detail_prompt = " Give extensive details." if style == "detailed" else ""
        return types.GetPromptResult(
            description="Summarize the current spaces",
            messages=[
                types.PromptMessage(
                    role="user",
                    content=types.TextContent(
                        type="text",
                        text=f"Here are the current spaces to summarize:{detail_prompt}\n\n"
                        + "\n".join(
                            f"- {space['key']}: {space['name']} ({space['type']})"
                            for space in self.space_list()["results"]
                        ),
                    ),
                )
            ],
        )

    @mcp_tool()
    def space_list(
        self,
        start: int = 0,
        limit: int = 1000,
        expand: str = "metadata,icon,description,homepage",
        space_type=None,
        space_status="current",
        context: MCPContext | None = None,
    ) -> dict:
        """List spaces in Confluence.

        Args:
            start: The starting index of the returned spaces
            start: The starting index of the returned spaces (defaults to 0)
            limit: Maximum number of spaces to return (defaults to 1000)
            expand: A comma-separated list of properties to expand in the response (defaults to 'metadata,icon,description,homepage')
            space_type: Filter by space type (e.g., 'global', 'personal', defaults to None, i.e. returns all types)
            space_status: Filter by space status ('current' or 'archived', defaults to current)
            context: MCP context for the operation

        Returns:
            (dict): JSON response containing the spaces list under 'results' key
        """
        return self._api.get_all_spaces(
            start,
            limit,
            expand,
            space_type,
            space_status,
        )  # type: ignore

    @mcp_tool()
    def page_create(
        self,
        space: str,
        title: str,
        body: str,
        type: str = "page",
        parent_id: str | None = None,
        representation: str = "storage",
        editor: str | None = None,
        full_width: bool = False,
        status: str = "current",
        context: MCPContext | None = None,
    ):  # -> Response | Any | None:# -> Response | Any | None:# -> Response | Any | None:  # -> Response | Any | bytes | Any | None | str:
        """Create page in Confluence space.

        Args:
            space: The identifier of the Confluence space
            title: The title of the new page
            body: The content/body of the new page
            type: The type of content to create (defaults to 'page')
            parent_id: The ID of the parent page if this is a child page (defaults to None)
            representation: The representation of the content ('storage' or 'wiki', defaults to 'storage')
            editor: The editor to use for the page (defaults to None, alternative is 'v2')
            full_width: If to use full width layout (defaults to False)
            status: The status of the page (defaults to None, i.e. 'current')
        """
        return self._api.create_page(
            space,
            title,
            body,
            parent_id,
            type,
            representation,
            editor,
            full_width,
            status,
        )

    @mcp_tool()
    def page_get(
        self,
        page_id: str,
        status: str | None = None,
        expand: str | None = None,
        version: str | None = None,
        context: MCPContext | None = None,
    ):  # -> Response | Any | bytes | Any | None | str:
        """Get a specific Confluence page by its ID.

        Args:
            page_id: The ID of the page to retrieve
            status: Page status to retrieve ('current' or specific version, defaults to None, i.e. current version)
            expand: A comma-separated list of properties to expand in the response (defaults to None, i.e. 'history,space,version')
            version: Specific version number to retrieve (optional)
            context: MCP context for the operation

        Returns:
            (Any): JSON response containing the page details
        """
        return self._api.get_page_by_id(page_id, status, expand, version)

    @mcp_tool()
    def page_update(
        self,
        page_id: str,
        title: str,
        body: str,
        parent_id: str | None = None,
        type: str = "page",
        representation: str = "storage",
        minor_edit: bool = False,
        version_comment: str | None = None,
        always_update: bool = False,
        full_width: bool = False,
    ):
        """Update a Confluence page.

        Args:
            page_id: The ID of the page to update
            title: The new title for the page
            body: The new content/body for the page
            parent_id: The ID of the parent page if moving the page
            type: The type of content to update (defaults to 'page')
            representation: The representation of the content ('storage' or 'wiki', defaults to 'storage')
            minor_edit: Whether this is a minor edit (defaults to False)
            version_comment: Optional comment to describe the change
            always_update: Force update even if version conflict (defaults to False)
            full_width: If to use full width layout (defaults to False)

        Returns:
            (Any): JSON response containing the updated page details

        Notes:
            The 'storage' representation is the default Confluence storage format.
            The 'wiki' representation allows using wiki markup syntax.
        """
        return self._api.update_page(
            page_id,
            title,
            body,
            parent_id,
            type,
            representation,
            minor_edit,
            version_comment,
            always_update,
            full_width,
        )

    @mcp_tool()
    def page_delete(
        self,
        page_id: str,
        status: str | None = None,
        recursive: bool = False,
        context: MCPContext | None = None,
    ):
        """Delete a Confluence page.

        Args:
            page_id: The ID of the page to delete
            status: OPTIONAL: type of page
            recursive: if True - will recursively delete all children pages too (defaults to False)
            context: MCP context for the operation
        """
        return self._api.remove_page(page_id, status, recursive)

    @mcp_tool()
    def page_list(
        self,
        space_key: str,
        start: int = 0,
        limit: int = 1000,
        status: str | None = None,
        expand: str | None = None,
        content_type: str = "page",
        context: MCPContext | None = None,
    ):  # -> Any | Any:# -> Any | Any:
        """List pages in a Confluence space.

        Args:
            space_key: The key of the space to get pages from
            start: The starting index of the returned pages (defaults to 0)
            limit: Maximum number of pages to return (defaults to 1000)
            status: Filter by page status ('current' or 'archived', defaults to None, i.e. all pages)
            expand: A comma-separated list of properties to expand in the response (defaults to None, i.e. 'history,space,version')
            content_type: The type of content to return (defaults to 'page')
            context: MCP context for the operation

        Returns:
            (Any): List of pages in the specified space
        """
        return self._api.get_all_pages_from_space(
            space_key,
            start,
            limit,
            status,
            expand,
            content_type,
        )



import pytest
from typer.testing import CliRunner

from starbridge.cli import cli


@pytest.fixture
def runner():
    return CliRunner()


def test_mcp_tools(runner):
    """Check tools include listing spaces and creating pages."""
    result = runner.invoke(cli, ["confluence", "mcp", "tools"])
    assert result.exit_code == 0
    assert "name='starbridge_confluence_info'" in result.stdout
    assert "name='starbridge_confluence_page_create'" in result.stdout
    assert "name='starbridge_confluence_space_list'" in result.stdout



"""
CLI to interact with Hello World
"""

import os
import subprocess
import sys
import tempfile
from pathlib import Path
from typing import Annotated

import typer

from starbridge.utils.console import console

from .service import Service

cli = typer.Typer(no_args_is_help=True)


@cli.command()
def health():
    """Health of Hello World"""
    console.print(Service().health())


@cli.command()
def info():
    """Info about Hello World"""
    console.print(Service().info())


@cli.command()
def hello():
    """Print Hello World!"""
    console.print(Service().hello())


@cli.command()
def bridge(
    dump: Annotated[
        bool,
        typer.Option(
            help="If set, will dump to file starbridge.png in current working directory. Defaults to opening viewer to show the image."
        ),
    ] = False,
) -> None:
    """Show image of starbridge"""
    image = Service().bridge()
    if dump:
        image.save("starbridge.png")
    else:
        image.show()


@cli.command()
def pdf(
    dump: Annotated[
        bool,
        typer.Option(
            help="If set, will dump to file starbridge.pdf in current working directory. Defaults to opening viewer to show the document."
        ),
    ] = False,
) -> None:
    """Show image of starbridge"""
    pdf = Service().pdf_bytes()

    if dump:
        pdf_path = Path("starbridge.pdf")
        pdf_path.write_bytes(pdf)
    else:
        # Create temporary file that gets auto-deleted when closed
        with tempfile.NamedTemporaryFile(suffix=".pdf", delete=False) as tmp:
            tmp.write(pdf)
            tmp_path = Path(tmp.name)
            try:
                if sys.platform == "darwin":  # macOS
                    subprocess.run(["open", tmp_path], check=True)
                elif sys.platform == "win32":  # Windows
                    os.startfile(tmp_path)  # type: ignore
                else:  # Linux and others
                    subprocess.run(["xdg-open", tmp_path], check=True)

                # Give the viewer some time to open the file
                import time

                time.sleep(2)
            finally:
                # Clean up temp file after viewer has opened it
                tmp_path.unlink(missing_ok=True)



"""
CLI to interact with Confluence
"""

from typing import Annotated

import typer

from starbridge.utils.console import console

from .service import Service

cli = typer.Typer(no_args_is_help=True)


@cli.command()
def health():
    """Health of Confluence"""
    console.print(Service().health())


@cli.command()
def info():
    """Info about Confluence"""
    console.print(Service().info())


cli_mcp = typer.Typer(no_args_is_help=True)
cli.add_typer(cli_mcp, name="mcp")


@cli_mcp.callback()
def mcp():
    """MCP endpoints"""


@cli_mcp.command()
def tools():
    """List tools exposed via MCP"""
    console.print(Service().tool_list())


@cli_mcp.command()
def resources():
    """List resources exposed via MCP"""
    console.print(Service().resource_list())


@cli_mcp.command()
def resource_types():
    """List resources exposed via MCP"""
    console.print(Service().resource_type_list())


@cli_mcp.command(name="space")
def resource_space(
    key: Annotated[
        str, typer.Argument(help="Key of the space to retrieve as resource")
    ],
):
    """Get space resource as exposed via MCP"""
    console.print(Service().space_get(key))


@cli_mcp.command()
def prompts():
    """List prompts exposed via MCP"""
    console.print(Service().prompt_list())


@cli_mcp.command(name="space-summary")
def prompt_space_summary(
    style: Annotated[str, typer.Option(help="Style of summary")] = "brief",
):
    """Prompt to generate summary of spaces"""
    console.print(Service().space_summary(style))


cli_space = typer.Typer(no_args_is_help=True)
cli.add_typer(cli_space, name="space")


@cli_space.callback()
def space():
    """Operations on Confluence spaces"""


@cli_space.command(name="list")
def space_list():
    """Get info about all space"""
    console.print(Service().space_list())


cli_page = typer.Typer(no_args_is_help=True)
cli.add_typer(cli_page, name="page")


@cli_page.callback()
def page():
    """Operations on Confluence pages"""


@cli_page.command(name="list")
def page_list(space_key: str = typer.Option(..., help="Space key")):
    """List pages in a space"""
    console.print(Service().page_list(space_key))


@cli_page.command(name="create")
def page_create(
    space_key: str = typer.Option(..., help="Space key"),
    title: str = typer.Option(..., help="Title of the page"),
    body: str = typer.Option(..., help="Body of the page"),
    page_id: str = typer.Option(None, help="Parent page id"),
):
    """Create a new page"""
    console.print(Service().page_create(space_key, title, body, page_id))


@cli_page.command(name="read")
def page_get(
    page_id: str = typer.Option(None, help="Parent page id"),
):
    """Read a page"""
    console.print(Service().page_get(page_id))


@cli_page.command(name="update")
def page_update(
    page_id: str = typer.Option(..., help="Pager id"),
    title: str = typer.Option(..., help="Title of the page"),
    body: str = typer.Option(..., help="Body of the page"),
):
    """Update a page"""
    console.print(Service().page_update(page_id, title, body))


@cli_page.command(name="delete")
def page_delete(page_id: str = typer.Option(..., help="Pager id")):
    """Delete a page"""
    console.print(Service().page_delete(page_id))



import json
import platform
import subprocess
import sys
import time
from pathlib import Path

import psutil
import typer

from starbridge.base import __project_name__
from starbridge.mcp import MCPBaseService, MCPContext, mcp_tool

from . import cli


class Service(MCPBaseService):
    """Service class for Claude operations."""

    def __init__(self):
        super().__init__()

    @staticmethod
    def get_cli() -> tuple[str | None, typer.Typer | None]:
        """Get CLI for Claude service."""
        return "claude", cli.cli  # type: ignore

    @mcp_tool()
    def health(self, context: MCPContext | None = None) -> str:
        """Check if Claude Desktop application is installed and is running."""
        if not self.is_installed():
            return "DOWN: Not installed"
        if not self.is_running():
            return "DOWN: Not running"
        return "UP"

    @mcp_tool()
    def info(self, context: MCPContext | None = None):
        """Get info about Claude Desktop application."""
        data = {
            "is_installed": self.is_installed(),
            "application_directory": None,
            "config_path": None,
            "log_path": None,
            "config": None,
            "processes": [],
        }
        if self.is_installed():
            data["application_directory"] = str(self.application_directory())
            if self.has_config():
                data["config_path"] = str(self.config_path())
                data["config"] = self.config_read()
                data["log_path"] = str(self.log_path())
        # Add list of running node processes
        data["processes"] = [
            proc.info for proc in psutil.process_iter(attrs=["pid", "name"])
        ]
        return data

    @mcp_tool()
    def restart(self, context: MCPContext | None = None):
        """Restart Claude Desktop application."""
        self._restart()
        return "Claude Desktop application restarted"

    @staticmethod
    def _get_starbridge_container_claude_path() -> Path:
        return Path("/Claude")

    @staticmethod
    def is_running_in_starbridge_container() -> bool:
        return Service._get_starbridge_container_claude_path().is_dir()

    @staticmethod
    def application_directory() -> Path:
        """Get path of Claude config directory based on platform."""
        # Check if running in starbridge container with mounted /Claude directory
        if Service.is_running_in_starbridge_container():
            return Service._get_starbridge_container_claude_path()

        # Regular platform-specific paths
        if sys.platform == "darwin":
            return Path(Path.home(), "Library", "Application Support", "Claude")
        elif sys.platform == "win32":
            return Path(Path.home(), "AppData", "Roaming", "Claude")
        elif sys.platform == "linux":
            return Path(Path.home(), ".config", "Claude")
        raise RuntimeError(f"Unsupported platform {sys.platform}")

    # Move all the static methods from Application
    @staticmethod
    def is_installed() -> bool:
        """Check if Claude Desktop application is installed."""
        return Service.application_directory().is_dir()

    @staticmethod
    def is_running() -> bool:
        """Check if Claude Desktop application is running."""
        if platform.system() != "Darwin":
            raise RuntimeError("This command only works on macOS")

        class Result:
            def __init__(self, returncode):
                self.returncode = returncode

        process_found = any(
            proc.info["name"] == "Claude"
            for proc in psutil.process_iter(attrs=["name"])
        )

        ps_check = Result(0 if process_found else 1)
        return ps_check.returncode == 0

    @staticmethod
    def config_path() -> Path:
        """Get path of Claude config based on platform."""
        return Service.application_directory() / "claude_desktop_config.json"

    @staticmethod
    def has_config() -> bool:
        """Check if Claud has configuration."""
        return Service.config_path().is_file()

    @staticmethod
    def config_read() -> dict:
        """Read config from file."""
        config_path = Service.config_path()
        if config_path.is_file():
            with open(config_path, encoding="utf8") as file:
                return json.load(file)
        raise FileNotFoundError(f"No config file found at '{config_path}'")

    @staticmethod
    def config_write(config: dict) -> dict:
        """Write config to file."""
        config_path = Service.config_path()
        with open(config_path, "w", encoding="utf8") as file:
            json.dump(config, file, indent=2)
        return config

    @staticmethod
    def log_directory() -> Path:
        """Get path of Claude log directory based on platform."""
        if sys.platform == "darwin":
            return Path(
                Path.home(),
                "Library",
                "Logs",
                "Claude",
            )
        elif sys.platform == "win32":
            return Path(
                Path.home(),
                "AppData",
                "Roaming",
                "Claude",
                "logs",
            )
        elif sys.platform == "linux":
            return Path(
                Path.home(),
                ".logs",
                "Claude",
            )
        raise RuntimeError(f"Unsupported platform {sys.platform}")

    @staticmethod
    def log_path(mcp_server_name: str | None = __project_name__) -> Path:
        """Get path of mcp ."""
        path = Service.log_directory()
        if mcp_server_name is None:
            return path / "mcp.log"
        return path / f"mcp-server-{mcp_server_name}.log"

    @staticmethod
    def install_mcp_server(
        mcp_server_config: dict, mcp_server_name=__project_name__, restart=True
    ) -> bool:
        """Install MCP server in Claude Desktop application."""
        if Service.is_installed() is False:
            raise RuntimeError(
                f"Claude Desktop application is not installed at '{Service.application_directory()}'"
            )
        try:
            config = Service.config_read()
        except FileNotFoundError:
            config = {"mcpServers": {}}

        if (
            mcp_server_name in config["mcpServers"]
            and config["mcpServers"][mcp_server_name] == mcp_server_config
        ):
            if restart:
                Service._restart()
            return False

        config["mcpServers"][mcp_server_name] = mcp_server_config
        Service.config_write(config)
        if restart:
            Service._restart()
        return True

    @staticmethod
    def uninstall_mcp_server(
        mcp_server_name: str = __project_name__, restart=True
    ) -> bool:
        """Uninstall MCP server from Claude Desktop application."""
        if Service.is_installed() is False:
            raise RuntimeError(
                f"Claude Desktop application is not installed at '{Service.application_directory()}'"
            )
        try:
            config = Service.config_read()
        except FileNotFoundError:
            config = {"mcpServers": {}}
        if "name" not in config["mcpServers"]:
            return False
        del config["mcpServers"][mcp_server_name]
        Service.config_write(config)
        if restart:
            Service._restart()
        return True

    @staticmethod
    def platform_supports_restart():
        return platform.system() == "Darwin"

    @staticmethod
    def _restart():
        """Restarts the Claude desktop application on macOS."""
        if Service.platform_supports_restart() is False:
            raise RuntimeError("Restart of Claude not supported on this platform")

        ps_check = subprocess.run(
            ["pgrep", "-x", "Claude"], capture_output=True, text=True, check=False
        )

        if ps_check.returncode == 0:
            subprocess.run(["pkill", "-x", "Claude"], check=False)
            time.sleep(1)

        subprocess.run(["open", "-a", "Claude"], check=True)

    @staticmethod
    def _run_brew_command(args: list) -> tuple[int, str, str]:
        """Run a homebrew command and return (returncode, stdout, stderr)"""
        process = subprocess.run(
            ["brew"] + args, capture_output=True, text=True, check=False
        )
        return process.returncode, process.stdout, process.stderr

    @staticmethod
    def install_via_brew() -> bool:
        """Install Claude via Homebrew if not already installed."""
        if platform.system() != "Darwin":
            raise RuntimeError("Homebrew installation only supported on macOS")

        # Check if already installed
        returncode, _, _ = Service._run_brew_command(["list", "--cask", "claude"])
        if returncode == 0:
            return False  # Already installed

        # Install Claude
        returncode, _, stderr = Service._run_brew_command([
            "install",
            "--cask",
            "claude",
        ])
        if returncode != 0:
            raise RuntimeError(f"Failed to install Claude: {stderr}")

        return True

    @staticmethod
    def uninstall_via_brew() -> bool:
        """Uninstall Claude via Homebrew."""
        if platform.system() != "Darwin":
            raise RuntimeError("Homebrew uninstallation only supported on macOS")

        # Check if installed
        returncode, _, _ = Service._run_brew_command(["list", "--cask", "claude"])
        if returncode != 0:
            return False  # Not installed

        # Uninstall Claude
        returncode, _, stderr = Service._run_brew_command([
            "uninstall",
            "--cask",
            "claude",
        ])
        if returncode != 0:
            raise RuntimeError(f"Failed to uninstall Claude: {stderr}")

        return True



import logging
from typing import Literal

import click
import logfire
from pydantic_settings import BaseSettings, SettingsConfigDict
from rich.console import Console
from rich.logging import RichHandler

from starbridge.base import __project_name__
from starbridge.instrumentation import logfire_initialize
from starbridge.utils.settings import load_settings


def get_logger(name: str | None) -> logging.Logger:
    if (name is None) or (name == __project_name__):
        return logging.getLogger(__project_name__)
    return logging.getLogger(f"{__project_name__}.{name}")


class LoggingSettings(BaseSettings):
    model_config = SettingsConfigDict(
        env_prefix=f"{__project_name__.upper()}_LOGGING_",
        extra="ignore",
        env_file=".env",
        env_file_encoding="utf-8",
    )

    loglevel: Literal["FATAL", "CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG"] = "INFO"
    log_file_enabled: bool = True
    log_file_name: str = "starbridge.log"
    log_console_enabled: bool = False


settings = load_settings(LoggingSettings)

handlers = []

if settings.log_file_enabled:
    handlers.append(
        logging.FileHandler(settings.log_file_name),
    )

if settings.log_console_enabled:
    rich_handler = RichHandler(
        console=Console(stderr=True),
        markup=True,
        rich_tracebacks=True,
        tracebacks_suppress=[click],
        show_path=True,
        show_level=True,
        enable_link_path=True,
    )

    class CustomFilter(logging.Filter):
        def filter(self, record):
            return True

    rich_handler.addFilter(CustomFilter())
    handlers.append(rich_handler)

logfire_initialized = logfire_initialize()
if logfire_initialized:
    handlers.append(
        logfire.LogfireLoggingHandler(),
    )


logging.basicConfig(
    level=settings.loglevel,
    format="%(name)s: %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
    handlers=handlers,
)



import json
from pathlib import Path

import nox

nox.options.reuse_existing_virtualenvs = True
nox.options.default_venv_backend = "uv"


@nox.session(python=["3.11", "3.12", "3.13", "3.14"])
def test(session: nox.Session):
    session.install("-e .[dev]")
    session.run(
        "pytest",
        "--disable-warnings",
        "--junitxml=junit.xml",
        "--cov=starbridge",
        "--cov-report=term-missing",
        "--cov-report=html:coverage_html",
        "--cov-report=xml:coverage.xml",
    )


@nox.session(python=["3.11"])
def lint(session: nox.Session):
    session.install("-e .[dev]")
    session.run("ruff", "check", ".")
    session.run(
        "ruff",
        "format",
        "--check",
        ".",
    )


@nox.session(python=["3.11"])
def audit(session: nox.Session):
    session.install("-e .[dev]")
    session.run("pip-audit", "-f", "json", "-o", "vulnerabilities.json")
    session.run("jq", ".", "vulnerabilities.json", external=True)
    session.run("pip-licenses", "--format=json", "--output-file=licenses.json")
    session.run("jq", ".", "licenses.json", external=True)
    # Read and parse licenses.json
    licenses_data = json.loads(Path("licenses.json").read_text(encoding="utf-8"))

    licenses_inverted: dict[str, list[dict[str, str]]] = {}
    licenses_inverted = {}
    for pkg in licenses_data:
        license_name = pkg["License"]
        package_info = {"Name": pkg["Name"], "Version": pkg["Version"]}

        if license_name not in licenses_inverted:
            licenses_inverted[license_name] = []
        licenses_inverted[license_name].append(package_info)

    # Write inverted data
    Path("licenses-inverted.json").write_text(
        json.dumps(licenses_inverted, indent=2), encoding="utf-8"
    )
    session.run("jq", ".", "licenses-inverted.json", external=True)
    session.run("cyclonedx-py", "environment", "-o", "sbom.json")
    session.run("jq", ".", "sbom.json", external=True)



from pydantic import AnyUrl
from pydantic_settings import BaseSettings, SettingsConfigDict


class AtlassianSettings(BaseSettings):
    model_config = SettingsConfigDict(
        env_prefix="STARBRIDGE_ATLASSIAN_",
        extra="ignore",
        env_file=".env",
        env_file_encoding="utf-8",
    )

    url: AnyUrl
    email_address: str
    api_token: str



import os
import pathlib
import sys
from typing import Annotated, Any

import typer
from dotenv import dotenv_values
from rich.prompt import Prompt

import starbridge.claude
import starbridge.mcp
from starbridge.base import __project_name__, __version__
from starbridge.mcp import MCPBaseService, MCPServer
from starbridge.utils import console, get_logger

# Initializes logging and instrumentation
logger = get_logger(__name__)

logger.debug(f"Booting version: {__version__}")


cli = typer.Typer(
    name="Starbridge CLI",
    help=f"Starbride (Version: {__version__})",
    epilog="Built with love in Berlin by Helmut Hoffer von Ankershoffen",
)


@cli.callback(invoke_without_command=True)
def main(ctx: typer.Context):
    """Run MCP Server - alias for 'mcp serve'"""
    if ctx.invoked_subcommand is None:
        MCPServer.serve()


@cli.command()
def health():
    """Check health Starbridge, services, and their dependencies."""
    console.print(MCPServer().health())


@cli.command()
def info():
    """Info about Starbridge"""
    data: dict[str, Any] = {
        "version": __version__,
        "path": _get_starbridge_path(),
        "development_mode": _is_development_mode(),
        "env": _get_starbridge_env(),
    }

    # Auto-discover and get info from all services
    for service_class in MCPBaseService.get_services():
        service = service_class()
        service_name = service.__class__.__module__.split(".")[1]
        data[service_name] = service.info()

    console.print(data)
    logger.debug(data)


@cli.command()
def configure():
    """Generate .env file for Starbridge"""
    if not _is_development_mode():
        raise Exception("This command is only available in development mode")

    starbridge_path = pathlib.Path(_get_starbridge_path())
    env_example_path = starbridge_path / ".env.example"
    env_path = starbridge_path / ".env"

    if not env_example_path.exists():
        raise Exception(".env.example file not found")

    example_values = dotenv_values(env_example_path)
    current_values = dotenv_values(env_path) if env_path.exists() else {}

    new_values = {}
    for key in example_values:
        default_value = current_values.get(key, example_values[key])
        value = Prompt.ask(
            f"Enter value for {key}",
            default=default_value if default_value else None,
            password="TOKEN" in key or "SECRET" in key,
        )
        new_values[key] = value

    with open(env_path, "w") as f:
        for key, value in new_values.items():
            # Try to convert to number, if it fails, it's not a number
            try:
                float(value)
                f.write(f"{key}={value}\n")
            except ValueError:
                f.write(f'{key}="{value}"\n')


@cli.command()
def install(
    atlassian_url: Annotated[
        str,
        typer.Option(
            prompt="Base url of your Confluence and Jira installation",
            help="Base url of your Confluence and Jira installation, e.g. https://your-domain.atlassian.net",
        ),
    ] = os.environ.get("STARBRIDGE_ATLASSIAN_URL", "https://your-domain.atlassian.net"),
    atlassian_email_address: Annotated[
        str,
        typer.Option(
            prompt="Email address of your Atlassian account",
            help="Email address of your Atlassian account, e.g. you@your-domain.com",
        ),
    ] = os.environ.get("STARBRIDGE_ATLASSIAN_EMAIL_ADDRESS", "you@your-domain.com"),
    atlassian_api_token: Annotated[
        str,
        typer.Option(
            prompt="Go to https://id.atlassian.com/manage-profile/security/api-tokens to create an API token for starbridge",
            help="API token of your Atlassian account, go to https://id.atlassian.com/manage-profile/security/api-tokens to create one",
        ),
    ] = os.environ.get("STARBRIDGE_ATLASSIAN_API_TOKEN", "YOUR_TOKEN"),
    restart_claude: Annotated[
        bool,
        typer.Option(
            help="Restart Claude Desktop application post installation",
        ),
    ] = starbridge.claude.Service.platform_supports_restart(),
    image: Annotated[
        str,
        typer.Option(
            help="Docker image to use for Starbridge. Only applies if started as container.",
        ),
    ] = "helmuthva/starbridge:latest",
):
    """Install starbridge within Claude Desktop application by adding to configuration and restarting Claude Desktop app"""
    if starbridge.claude.Service.install_mcp_server(
        _generate_mcp_server_config(
            atlassian_url, atlassian_email_address, atlassian_api_token, image
        ),
        restart=restart_claude,
    ):
        console.print("Starbridge installed with Claude Desktop application.")
        if not restart_claude:
            console.print(
                "Please restart Claude Desktop application to complete the installation."
            )

    else:
        console.print("Starbridge was already installed", style="warning")


@cli.command()
def uninstall():
    """Install starbridge from Claude Desktop application by removing from configuration and restarting Claude Desktop app"""
    if starbridge.claude.Service.uninstall_mcp_server():
        console.print("Starbridge uninstalled from Claude Destkop application.")
    else:
        console.print("Starbridge was no installed", style="warning")


def _is_development_mode():
    return "uvx" not in sys.argv[0].lower()


def _get_starbridge_path() -> str:
    return str(pathlib.Path(__file__).parent.parent.parent)


def _get_starbridge_env():
    """Get environment variables starting with STARBRIDGE_"""
    return {k: v for k, v in os.environ.items() if k.startswith("STARBRIDGE_")}


def _generate_mcp_server_config(
    atlassian_url: str,
    atlassian_email_address: str,
    atlassian_api_token: str,
    image: str = "helmuthva/starbridge:latest",
) -> dict:
    """Generate configuration file for Starbridge"""
    env = {
        "STARBRIDGE_ATLASSIAN_URL": atlassian_url,
        "STARBRIDGE_ATLASSIAN_EMAIL_ADDRESS": atlassian_email_address,
        "STARBRIDGE_ATLASSIAN_API_TOKEN": atlassian_api_token,
    }
    if starbridge.claude.Service.is_running_in_starbridge_container():
        return {
            "command": "docker",
            "args": [
                "run",
                "-i",
                "--rm",
                "-e",
                "STARBRIDGE_ATLASSIAN_URL",
                "-e",
                "STARBRIDGE_ATLASSIAN_EMAIL_ADDRESS",
                "-e",
                "STARBRIDGE_ATLASSIAN_API_TOKEN",
                image,
            ],
            "env": env,
        }
    if _is_development_mode():
        return {
            "command": "uv",
            "args": [
                "--directory",
                _get_starbridge_path(),
                "run",
                __project_name__,
            ],
            "env": env,
        }
    return {
        "command": "uvx",
        "args": [
            __project_name__,
        ],
        "env": env,
    }


cli.add_typer(
    starbridge.mcp.cli,
    name="mcp",
    help="MCP operations",
)

for service_class in MCPBaseService.get_services():
    name, typer_cli = service_class.get_cli()
    if name and typer_cli:
        cli.add_typer(
            typer_cli,
            name=name,
            help=f"{name.title()} operations",
        )

if __name__ == "__main__":
    try:
        cli()
    except Exception as e:
        logger.error(e)
